@@ -31,10 +31,12 @@ def enter_and_leave_function(func):
 
 
 class CloudPods:
-    def __init__(self,keystone_url,username,password):
+    def __init__(self,keystone_url,username,password,domain="default",project="system"):
         self.__keystone_url = keystone_url
         self.__username = username
         self.__password = password
+        self.__domain = domain
+        self.__project = project
         self.__session = self.__get_session()
         self.__endpoints = self.__get_endpoint()
         self.__compute_url = f"{self.__endpoints['region2']}/"
@@ -60,8 +62,7 @@ class CloudPods:
             "User-Agent": "yunioncloud-go/201708"
         }
         session.headers.update(headers)
-        get_token_url = "/auth/tokens"
-        url = self.__keystone_url + get_token_url
+        url = self.__keystone_url + "/auth/tokens"
         data = {
             "auth": {
                 "context": {
@@ -81,9 +82,9 @@ class CloudPods:
                 "scope": {
                     "project": {
                         "domain": {
-                            "name": "default"
+                            "name": self.__domain
                         },
-                        "name": "system"
+                        "name": self.__project
                     }
                 }
             }
@@ -93,7 +94,13 @@ class CloudPods:
             if rs.status_code != 200:
                 log.error(f"Failed to get /auth/tokens: Error. err msg is {str(rs.text)}")
                 return None
+            if "X-Subject-Token" not in rs.headers.keys():
+                log.error(f"Failed to get /auth/tokens for X-Subject-Token not in rs.headers.keys(): Error. err msg is {str(rs.text)}")
+                return None
             token = rs.headers["X-Subject-Token"]
+            if not token:
+                log.error(f"Failed to get /auth/tokens for token is None: Error. err msg is {str(rs.text)}")
+                return None
             log.info(f"Get /auth/tokens success. token is {token}")
             headers['X-Auth-Token'] = token
             session.headers.update(headers)
@@ -111,9 +118,22 @@ class CloudPods:
             if rs.status_code != 200:
                 log.error(f"Failed to get /endpoints: Error. err msg is {str(rs.text)}")
                 return None
-            rs_data = rs.json()
+            try:
+                rs_data = rs.json()
+            except Exception as e:
+                log.error(f"Failed to get /endpoints: Error. err msg is {str(e)}", exc_info=True)
+                return None
             endpoints=dict({})
+            if "endpoints" not in rs_data.keys():
+                log.error(f"Failed to get /endpoints for endpoints not in rs_data.keys(): Error. rs.text is {str(rs.text)}")
+                return None
             for elem in rs_data["endpoints"]:
+                if "service_name" not in elem.keys():
+                    log.error(f"Failed to get /endpoints for service_name not in elem.keys(): Error. rs.text is {str(rs.text)}")
+                    return None
+                if "url" not in elem.keys():
+                    log.error(f"Failed to get /endpoints for url not in elem.keys(): Error. rs.text is {str(rs.text)}")
+                    return None
                 if elem["service_name"] not in endpoints.keys():
                     endpoints[elem["service_name"]]=elem["url"]
             return endpoints
@@ -123,8 +143,7 @@ class CloudPods:
     @enter_and_leave_function
     def __create_server(self, server, count=1):
         try:
-            create_server_url = "/servers"
-            url = self.__compute_url + create_server_url
+            url = self.__compute_url + "/servers"
             data = {
                 "count": count,
                 "server": server
@@ -133,9 +152,14 @@ class CloudPods:
             if rs.status_code != 200:
                 log.error(f"Failed to create server: Error. err msg is {str(rs.text)}")
                 return None
-            return rs.json()
+            try:
+                rs_data = rs.json()
+                return rs_data
+            except Exception as e:
+                log.error(f"Failed to create server: Error. err msg is {str(e)}")
+                return None
         except Exception as e:
-            log.error(f"Failed to create server: Error. err msg is {str(e)}",exc_info=True)
+            log.error(f"Failed to create server: Error. err msg is {str(e)}")
             return None
 
     @enter_and_leave_function
@@ -193,33 +217,41 @@ class CloudPods:
                 server_ids.append(elem["body"]["id"])
             return server_ids
         else:
+            log.warn(f"Failed to create server for get no server_id, res is {res}")
             return server_ids
 
     @enter_and_leave_function
     def get_server_detail(self, server_id):
         try:
-            get_all_snapshots = f"/servers/{server_id}"
-            url = self.__compute_url + get_all_snapshots
+            url = self.__compute_url + f"/servers/{server_id}"
             rs = self.__session.get(url=url, verify=False, timeout=600)
-            rs_data = rs.json()
-            return rs_data
+            if rs.status_code != 200:
+                log.warn(f"Failed to request /servers/{server_id}: Error. err msg is {str(rs.text)}")
+                return None
+            try:
+                rs_data = rs.json()
+                return rs_data
+            except Exception as e:
+                log.warn(f"Failed to request /servers/{server_id}: Error. err msg is {str(e)}")
+                return None
         except Exception as e:
-            log.error(f"Failed to request /servers/{server_id}: Error.\rerr msg is {str(e)}",exc_info=True)
+            log.warn(f"Failed to request /servers/{server_id}: Error.\rerr msg is {str(e)}")
+            return None
 
     @enter_and_leave_function
     def get_server_ip(self, server_id):
         server = self.get_server_detail(server_id)
         if "server" not in server.keys():
-            log.error(f"failed get server info by server name {server_id}, server info is {server}.")
+            log.warn(f"failed get server info by server name {server_id}, server info is {server}.")
             return None
         if "nics" not in server["server"].keys():
-            log.error(f"nics not in server['server'], server['server'] is {server['server']}")
+            log.warn(f"nics not in server['server'], server['server'] is {server['server']}")
             return None
         if len(server["server"]["nics"])<1:
-            log.error(f"server['server']['nics'] has no elem, its length is 0")
+            log.warn(f"server['server']['nics'] has no elem, its length is 0")
             return None
         if "ip_addr" not in server["server"]["nics"][0].keys():
-            log.error(f"ip_addr do not in server['server']['nics'][0], server['server']['nics'][0] is {server['server']['nics'][0]}")
+            log.warn(f"ip_addr do not in server['server']['nics'][0], server['server']['nics'][0] is {server['server']['nics'][0]}")
             return None
         return server["server"]["nics"][0]["ip_addr"]
 
@@ -229,23 +261,30 @@ class CloudPods:
         running_status_count = 0
         while True:
             if time_cost>=timeout:
-                log.error(f"Timeout Error: {timeout} seconds passed but server still not online: Error.")
+                log.warn(f"Timeout Error: {timeout} seconds passed but server still not online: Error.")
                 return False
             server_detail = self.get_server_detail(server_id)
             if "server" in server_detail.keys() and "status" in server_detail["server"].keys():
                 if server_detail["server"]["status"] == "disk_fail":
+                    log.warn(f"server_id: {server_id} disk_fail")
                     return False
                 if server_detail["server"]["status"] == "deploy_fail":
+                    log.warn(f"server_id: {server_id} deploy_fail")
                     return False
                 if server_detail["server"]["status"] == "ready":
+                    log.warn(f"server_id: {server_id} ready")
                     return False
                 if "_fail" in server_detail["server"]["status"]:
+                    log.warn(f"server_id: {server_id} {server_detail['server']['status']}")
                     return False
                 if server_detail["server"]["status"] == "running":
+                    log.info(f"server_id: {server_id} running")
                     running_status_count += 1
                     if running_status_count > 5:
+                        log.info(f"server_id: {server_id} running for 5 times check.")
                         return True
                     else:
+                        log.warn(f"just {running_status_count} times check: server_id: {server_id} running status: {server_detail['server']['status']}")
                         time.sleep(1)
                         time_cost+=1
                         continue
@@ -254,7 +293,7 @@ class CloudPods:
                 time.sleep(5)
                 time_cost+=5
             else:
-                log.error(f"Failed to get server_detail: Error. server_detail is: {server_detail}")
+                log.warn(f"Failed to get server_detail: Error. server_detail is: {server_detail}")
                 return False
 
     @enter_and_leave_function
@@ -271,8 +310,15 @@ class CloudPods:
                 "summary_stats": True
             }
             rs = self.__session.get(url=url, params=payload, verify=False, timeout=600)
-            rs_data = rs.json()
-            return bool(rs_data["servers"])
+            try:
+                rs_data = rs.json()
+                if "servers" not in rs_data.keys():
+                    log.warn(f"Failed to get server_detail: Error. server_detail is: {rs_data}")
+                    return False
+                return bool(rs_data["servers"])
+            except Exception as e:
+                log.warn(f"Failed to get server_detail: Error. server_detail is: {str(e)}")
+                return False
         except Exception as e:
             log.error(f"Failed to request /servers: Error.\rerr msg is {str(e)}",exc_info=True)
             return False
@@ -289,8 +335,12 @@ class CloudPods:
                 "summary_stats": True
             }
             rs = self.__session.get(url=url, params=payload, verify=False, timeout=600)
-            rs_data = rs.json()
-            return rs_data
+            try:
+                rs_data = rs.json()
+                return rs_data
+            except Exception as e:
+                log.warn(f"Failed to get host spec: Error. rs is {rs.text}.\rerr msg is {str(e)}",exc_info=True)
+                return None
         except Exception as e:
             log.error(f"Failed to request /hosts/{host_id}/spec: Error.\rerr msg is {str(e)}",exc_info=True)
             return None
@@ -308,10 +358,15 @@ class CloudPods:
                 "summary_stats":  True
             }
             rs = self.__session.get(url=url, params=payload, verify=False, timeout=600)
-            rs_data = rs.json()
-            return rs_data
+            try:
+                rs_data = rs.json()
+                return rs_data
+            except Exception as e:
+                log.warn(f"Failed to get baremetal hosts: Error. rs is {rs.text}.\rerr msg is {str(e)}",exc_info=True)
+                return None
         except Exception as e:
-            log.error(f"Failed to request /hosts: Error.\rerr msg is {str(e)}",exc_info=True)
+            log.warn(f"Failed to request /hosts: Error.\rerr msg is {str(e)}",exc_info=True)
+            return None
 
     @enter_and_leave_function
     def get_baremetal_servers(self):
@@ -326,10 +381,15 @@ class CloudPods:
                 "summary_stats": True
             }
             rs = self.__session.get(url=url, params=payload, verify=False, timeout=600)
-            rs_data = rs.json()
-            return rs_data
+            try:
+                rs_data = rs.json()
+                return rs_data
+            except Exception as e:
+                log.warn(f"Failed to get baremetal servers: Error. rs is {rs.text}.\rerr msg is {str(e)}",exc_info=True)
+                return None
         except Exception as e:
             log.error(f"Failed to request /servers: Error.\rerr msg is {str(e)}", exc_info=True)
+            return None
 
     @enter_and_leave_function
     def delete_server(self, server_id):
@@ -347,6 +407,7 @@ class CloudPods:
                 return False
         except Exception as e:
             log.error(f"Failed to delete server {server_id}: Error.\nerr msg is {str(e)}",exc_info=True)
+            return False
 
 
 if __name__ == '__main__':
